import os
import sys
import asyncio


def runner(fun, *args):
    """
    Generic asyncio.run() equivalent for Python >= 3.5
    """
    if os.name == "nt" and (3, 7) <= sys.version_info < (3, 8):
        asyncio.set_event_loop_policy(
            asyncio.WindowsProactorEventLoopPolicy()  # type: ignore
        )

    if sys.version_info >= (3, 7):
        result = asyncio.run(fun(*args))
    else:
        if os.name == "nt":
            loop = asyncio.ProactorEventLoop()
        else:
            loop = asyncio.new_event_loop()
            asyncio.get_child_watcher().attach_loop(loop)
        result = loop.run_until_complete(fun(*args))
        loop.close()

    return result



#!/usr/bin/env python
r"""
Recursively find files containing text.
This method is slower than grep or findstr, but is cross-platform and easier syntax.

## benchmarks:

time findtext xarray
18.6 sec

# note there are no "" on the command below. It's the equivalent of the defaults for the Python script.
time grep -r -l \
  --exclude-dir={\_site,\.git,\.eggs,build,dist,\.mypy_cache,.pytest_cache,*\.egg-info} \
  --include=*.{py,cfg,ini,txt,pdf,md,rst,tex,f,f90,for,f95,c,h,cpp,hpp,m} \
  xarray .
0.55 sec

---
time findtext xarray "*.py"
1.14 sec

time grep -r -l \
  --exclude-dir={\_site,\.git,\.eggs,build,dist,\.mypy_cache,.pytest_cache,*\.egg-info} \
  --include=*.py  xarray .
0.15 sec

"""
from pathlib import Path
import shutil
import subprocess
from argparse import ArgumentParser

import pyfindfiles.text as pf

EXCLUDEDIR = ["_site", ".git", ".eggs", "build", "dist", ".mypy_cache", ".pytest_cache"]

# from colorama--works for Unix Terminal, PowerShell and Windows Terminal
MAGENTA = "\x1b[45m"
BLACK = "\x1b[40m"


def main():
    p = ArgumentParser(description="searches for TEXT under DIR and echos back filenames")
    p.add_argument("txt", help="text to search for")  # required
    p.add_argument("globext", help="filename glob", nargs="?", default=pf.TXTEXT)
    p.add_argument("dir", help="root dir to search", nargs="?", default=".")
    p.add_argument("-c", "--run", help="command to run on files e.g. notepad++")
    p.add_argument("-e", "--exclude", help="exclude files/dirs", nargs="+", default=EXCLUDEDIR)
    p.add_argument("-v", "--verbose", action="store_true")
    P = p.parse_args()

    root = Path(P.dir).expanduser().resolve()
    if not root.is_dir():
        raise SystemExit("{} is not a directory.".format(root))

    files = pf.findtext(P.dir, P.txt, P.globext, P.exclude)

    flist = []
    if P.verbose:
        for fn, matches in files:
            flist.append(str(fn))
            print(MAGENTA + str(fn) + BLACK)
            for k, v in matches.items():
                print(k, ":", v)
    else:
        for fn, _ in files:
            flist.append(str(fn))
            print(fn)

    if not flist:
        raise SystemExit("no files found in {} matching {}".format(P.dir, P.txt))

    if P.run:
        exe = shutil.which(P.run)  # necessary for some Windows program e.g. VScode
        if not exe:
            raise SystemExit("could not find {}".format(exe))
        subprocess.run([exe] + flist)


if __name__ == "__main__":
    main()



#!/usr/bin/env python
"""
recursively find video files based on extension.
"""
import os
import logging
from pathlib import Path
import typing
import asyncio
from argparse import ArgumentParser
import itertools

import pyfindfiles.vid as fv
from pyfindfiles.runner import runner

VIDEXT = [".avi", ".mov", ".mp4", ".mpg", ".mpeg", ".webm", ".ogv", ".mkv", ".wmv"]


async def findvid_win(path: Path, exts: typing.Sequence[str]) -> typing.Iterator[Path]:
    futures = [fv.findvid_win(path, ext) for ext in exts]
    return itertools.chain.from_iterable(filter(None, await asyncio.gather(*futures)))


def main():
    p = ArgumentParser()
    p.add_argument("path", help="root path to start recursive search")
    p.add_argument("-v", "--verbose", action="store_true")
    p.add_argument("-ext", help="video extension to search for", nargs="+", default=VIDEXT)
    P = p.parse_args()

    root = Path(P.path).expanduser().resolve()
    if not root.is_dir():
        raise SystemExit("{} is not a directory.".format(root))

    if P.verbose:
        logging.basicConfig(level=logging.DEBUG)

    if os.name == "nt":
        videos = runner(findvid_win, root, P.ext)
    else:
        videos = fv.findvid_gnu(root, P.ext)

    for video in videos:
        print(video)


if __name__ == "__main__":
    main()



import os
from pathlib import Path
from typing import Dict, Iterable, Sequence, Tuple

TXTEXT = [
    "*.py",
    "*.cfg",
    "*.ini",
    "*.txt",
    "*.md",
    "*.rst",
    "*.tex",
    "*.build",
    "*.cmake",
    "*.f",
    "*.f90",
    "*.for",
    "*.f95",
    "*.c",
    "*.h",
    "*.cpp",
    "*.cxx",
    "*.cc",
    "*.hpp",
    "*.m",
]

BINEXT = ["*.pdf"]

MAXSIZE = 100e6  # arbitrary, bytes


def findtext(root: Path, txt: str, globext: Sequence[str], exclude: Sequence[str] = None) -> Iterable[Tuple[Path, Dict[int, str]]]:
    """
    multiple extensions with braces like Linux does not work in .rglob()
    """

    root = Path(root).expanduser()
    if not root.is_dir():
        raise NotADirectoryError(root)

    if isinstance(globext, (str, Path)):
        globext = [str(globext)]

    exc = set(exclude) if exclude else None

    for ext in globext:
        for fn in root.rglob(ext):
            if exc:
                excluded = exc.intersection(set(str(fn.resolve()).split(os.sep)))
                if excluded or not fn.is_file() or fn.stat().st_size > MAXSIZE:
                    continue

            with fn.open("r", encoding="utf8", errors="ignore") as f:
                matches = {i: str(line) for i, line in enumerate(f) if txt in line}

            if not matches:
                continue

            yield fn, matches



import typing
from pathlib import Path
import asyncio
import logging
import sys
import subprocess
import shutil


def findvid(path: Path, ext: typing.Sequence[str]) -> typing.List[Path]:
    """
    recursive file search in Pure Python.
    about 10 times slower than Linux find, but platform-independent.
    """
    flist = []  # type: typing.List[Path]

    path = Path(path).expanduser()

    for e in ext:
        flist += list(path.glob("**/*" + e))

    return flist


def findvid_gnu(path: Path, exts: typing.Sequence[str]) -> typing.List[str]:
    """
    recursive file search using GNU find
    """
    path = Path(path).expanduser()
    if isinstance(exts, str):
        exts = [exts]

    find = shutil.which("find")
    if not find:
        raise FileNotFoundError('could not find "find"')

    cmd = [find, str(path), "-type", "f"]

    if sys.platform != "darwin":
        cmd += ["-regextype", "posix-egrep"]

    cmd += ["-iregex", r".*(" + r"|".join(exts) + r")$"]

    logging.debug(" ".join(cmd))

    stdout = subprocess.check_output(cmd, universal_newlines=True).strip()

    return stdout.split("\n")


async def findvid_win(path: Path, ext: str) -> typing.List[Path]:
    """
    asynchronously find files with extension

    Parameters
    ----------

    path : pathlib.Path
        root directory to recursively search under
    ext : str
        file extension to look for

    Returns
    -------

    video: pathlib.Path
        path to video file
    """

    path = Path(path).expanduser()
    cmd = ["dir", "/s", "*" + ext]
    logging.debug(" ".join(cmd))
    # this has to be _shell due to that "dir" is part of Windows shell itself; _exec won't work.
    proc = await asyncio.create_subprocess_shell(
        " ".join(cmd), cwd=str(path), stdout=asyncio.subprocess.PIPE, stderr=asyncio.subprocess.DEVNULL,
    )
    stdout, _ = await proc.communicate()

    flist = []
    for r in stdout.decode("utf8").split("\n"):
        if not r:
            continue

        el = r.split()
        if not el:
            continue

        if el[0].startswith("Directory"):
            d = Path(" ".join(el[2:]))
            continue

        if el[-1].endswith(ext):
            flist.append(d / el[-1])

    return flist
